package com.community.huiju.controller;

import com.community.commom.mode.ResponseBean;
import com.community.commom.session.UserElement;
import com.community.huiju.common.base.BaseController;
import com.community.huiju.service.BuildingTreeServiceI;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cloud.context.config.annotation.RefreshScope;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpSession;

/**
 * @author FXF
 * @date 2019-04-01
 */
@RestController
@RefreshScope
@RequestMapping("/")
@Api(value = "楼栋树管理", description = "楼栋树管理 Api")
public class BuildingTreeController extends BaseController {
	
	@Autowired
	private BuildingTreeServiceI buildingTreeService;
	
	@ApiOperation(value = "上传楼栋信息接口", notes = "上传楼栋信息接口")
	@PostMapping(value = "/tree/uploadExcel", consumes = "multipart/*", headers = "content-type=multipart/form-data")
	public ResponseBean uploadExcel(@RequestParam("file") MultipartFile file) {
		ResponseBean responseBean = new ResponseBean();
		responseBean = buildingTreeService.getExcelData(file);
		return responseBean;
	}
	
	@ApiOperation(value = "上传楼栋信息并入库", notes = "上传楼栋信息并入库")
	@PostMapping(value = "/tree/submitExcel", consumes = "multipart/*", headers = "content-type=multipart/form-data")
	public ResponseBean submitExcel(@RequestParam("file") MultipartFile file, HttpSession session) {
		ResponseBean responseBean = new ResponseBean();
		UserElement userElement = getUserElement(session);
		responseBean = buildingTreeService.submitExcelData(file,userElement);
		return responseBean;
	}
	
	@ApiOperation(value = "获取树形数据", notes = "获取树形数据")
	@GetMapping(value = "/trees")
	public ResponseBean getTreeList(@RequestParam("id") Integer id,@RequestParam("treeType") String treeType,HttpSession session){
		ResponseBean responseBean = new ResponseBean();
		UserElement userElement = getUserElement(session);
		responseBean = buildingTreeService.getTreeList(userElement.getCommunityId(),id,treeType);
		return responseBean;
	}

	@ApiOperation(value = "添加当前节点", notes = "添加当前节点")
	@ApiImplicitParams({
			@ApiImplicitParam(paramType = "body", dataTypeClass = String.class, name = "parameter", value = "id:当前节点;" +
					"name:节点名称;type：类型，nodeNumber:输入的节点")
	})
	@RequestMapping(value = "/addnode",method = RequestMethod.POST)
	public ResponseBean addNode(@RequestBody String parameter, HttpSession session){
		UserElement userElement = getUserElement(session);
	    ResponseBean	responseBean = buildingTreeService.addNode(parameter, userElement.getCommunityId());
		return responseBean;
	}

	@ApiOperation(value = "删除当前节点", notes = "删除当前节点")
	@ApiImplicitParams({
			@ApiImplicitParam(paramType = "body", dataTypeClass = String.class, name = "parameter", value = "id:当前节点;" +
					"name:节点名称;type：类型，nodeNumber:输入的节点")
	})
	@RequestMapping(value = "/deletenode",method = RequestMethod.POST)
	public ResponseBean deletenode(@RequestBody String parameter, HttpSession session){
		UserElement userElement = getUserElement(session);
		ResponseBean	responseBean = buildingTreeService.deleteNode(parameter, userElement.getCommunityId());
		return responseBean;
	}
}
